;++++++++++++++++++++++++++++++++++++++++++++++++++
Func IsCLIParam($Switch, $Param)
  $SwitchLen = StringLen($Switch)

  If ($SwitchLen > StringLen($Param)) Then Return (0)
  $Temp = StringLeft ($Param,$SwitchLen)

  If ($Switch = $Temp) Then
    Return (1)
  Else
    Return (0)
  EndIf
EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
Func GetCLIParam($Switch, $Param)
  $SwitchLen = StringLen($Switch)
  If ($SwitchLen > StringLen($Param)) Then Return ("")
  Return(StringTrimLeft ($Param,$SwitchLen))
EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
Func ParseCLI (ByRef $VCF, ByRef $AVS, ByRef $UseFade, ByRef $FadeFPS)
  ; Local Init
  $VCF     = ""
  $AVS     = ""
  $UseFade = 1

  ; Obtain number of commandline parameters
  $ArgCount = $CmdLine[0]

  ; Return NoSuccess if
  If ($ArgCount = 0) Then Return (0)

  For $i = 1 To $ArgCount
    If (IsCLIParam("VCF=",      $CmdLine[$i]) <> 0) Then $VCF     = GetCLIParam("VCF=",      $CmdLine[$i])
    If (IsCLIParam("AVS=",      $CmdLine[$i]) <> 0) Then $AVS     = GetCLIParam("AVS=",      $CmdLine[$i])
    If (IsCLIParam("FADE=",     $CmdLine[$i]) <> 0) Then $UseFade = GetCLIParam("FADE=",     $CmdLine[$i])
    If (IsCLIParam("FADE-FPS=", $CmdLine[$i]) <> 0) Then $FadeFPS = GetCLIParam("FADE-FPS=", $CmdLine[$i])
  Next

  Return (1)
EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
Func ExtractCropping ($VCF,ByRef $Left, ByRef $Right, ByRef $Top, ByRef $Bottom)

  $HasCropping = 0

  $file = FileOpen($VCF, 0)

  While 1
    $line = FileReadLine($file)
    If @error = -1 Then ExitLoop
    If StringInStr($line,"VirtualDub.video.filters.instance[0].SetClipping(") Then
      $Temp   = StringSplit (StringTrimRight(StringReplace($Line,"VirtualDub.video.filters.instance[0].SetClipping(",""),2),",")
      $Left   = $Temp[1]
      $Top    = $Temp[2]
      $Right  = $Temp[3]
      $Bottom = $Temp[4]
      $HasCropping = 1
      $Temp = 0
    EndIf
  Wend

  FileClose($file)

  Return ($HasCropping)

EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
Func GenerateCutList ($VCF,$Fade)
  $MaxCuts = 4000
  Dim $CutMarks[$MaxCuts]

  For $i = 0 To $MaxCuts - 1
    $CutMarks[$i] = ""
  Next

  $CutList = ""
  $Count   = 0

  $file = FileOpen($VCF, 0)

  While 1
    $line = FileReadLine($file)
    If @error = -1 Then ExitLoop
    If StringInStr($line,"VirtualDub.subset.AddRange(") Then
      $Temp  = StringSplit (StringTrimRight(StringReplace($Line,"VirtualDub.subset.AddRange(",""),2),",")
      $FirstFrame = $Temp[1]
      $LastFrame  = $Temp[1] + $Temp[2] - 1
      $CutMarks[$Count] = "Trim(" & $FirstFrame & "," & $LastFrame & ")"
      $Count = $Count + 1
    EndIf
  Wend

  FileClose($file)

  For $i = 0 To $Count - 1
    If ($CutMarks[$i] <> "") Then
      ; Generate Cutlist without fading
      If NOT ($Fade = 1) Then
        If ($CutList <> "") Then $CutList = $CutList & "+" & $CutMarks[$i]
        If ($CutList = "")  Then $CutList = $CutMarks[$i]
      Else
      ; Generate Cutlist with fading
        If ($CutList <> "") Then $CutList = $CutList & "+" & $CutMarks[$i] & ".Reverse.FadeOut(FPS).Reverse.FadeOut2(FPS)"
        If ($CutList = "")  Then $CutList = $CutMarks[$i] & ".FadeOut2(FPS)"
      EndIf
    Else
      ExitLoop
    EndIf
  Next

  Return ($CutList)
EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
Func AddLine ($TextFile, $Line)
  $file = FileOpen($TextFile, 1)

  ; Check if file opened for reading OK
  If $file = -1 Then
    MsgBox(0, "Error", "Unable to open file.")
    Exit
  EndIf

  FileWriteLine($file, $Line)

  FileClose($file)

EndFunc
;++++++++++++++++++++++++++++++++++++++++++++++++++
; Variables Init
$Left    = 0
$Right   = 0
$Top     = 0
$Bottom  = 0
$Fade    = 1
$FadeFPS = 0
$VCF     = ""
$AVS     = ""

If (ParseCLI ($VCF, $AVS, $Fade, $FadeFPS) = 0) Then
  $VCF = FileOpenDialog("Choose VCF file" , "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}", "VirtualDub Config (*.VCF)", 1)
  $AVS = FileOpenDialog("Choose AVISynth script" , "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}", "AVISynth Script (*.AVS)", 2, StringTrimRight ($VCF,4) & ".AVS")

  $Button = MsgBox(36, "Fading:", "Do you to use fade in / out / crossfade for your cutpoints ?")

  If ($Button = 6) Then
    $Fade = 1
  Else
    $Fade = 0
  EndIf
EndIf

; If AVS is missing extension then add it
If (StringUpper(StringRight($AVS,4)) <> ".AVS") Then $AVS = $AVS & ".AVS"

AddLine ($AVS,@CRLF & "# Added by VCF2AVS")

$CutList = GenerateCutList ($VCF, $Fade)

$HasCropping = ExtractCropping ($VCF, $Left, $Right, $Top, $Bottom)

If ($Cutlist <> "") Then
  ; If no # of frames is specified for fading then a full second is used for every fade
  If ($Fade = 1) And ($FadeFPS=0)  Then AddLine ($AVS,"FPS=Round(Framerate)")
  ; If the # of frames is specified for fading then its added to the script
  If ($Fade = 1) And ($FadeFPS<>0) Then AddLine ($AVS,"FPS=" & $FadeFPS)
  ; Note that crossfading (blackout followed by blackin) always means the double
  ; amount of frames -> 2xFPS
  AddLine ($AVS, $Cutlist)
EndIf

If ($HasCropping = 1) Then
  AddLine ($AVS, "Top="    & $Top)
  AddLine ($AVS, "Bottom=" & $Bottom)
  AddLine ($AVS, "Left="   & $Left)
  AddLine ($AVS, "Right="  & $Right)
  AddLine ($AVS, "Crop(Left, Top, -Right, -Bottom)")
  AddLine ($AVS, "AddBorders(Left, Top, Right, Bottom)")
EndIf

If ($Fade = 1) Then AddLine ($AVS, "Reverse.FadeOut(2*FPS).Reverse.FadeOut2(2*FPS)")
